@@ -3,6 +3,7 @@
  *
  *  Authors: Laurent Dami            <laurent.d...@etat.ge.ch>
  *           Jean-Christophe Durand  <jean-christophe.d.....@etat.ge.ch>
+ *           Sbastien Cuendet       <sebastien.c.....@etat.ge.ch>
  *  LICENSE
  *  This library is free software, you can redistribute it and/or modify
  *  it under the same terms as Perl's artistic license.
@@ -10,7 +11,7 @@
  *--------------------------------------------------------------------------*/
 
 var GvaScript = {
-  Version: '1.10'
+  Version: '1.11'
 }
 
 //----------protoExtensions.js
@@ -1484,6 +1485,9 @@ GvaScript.AutoCompleter.prototype = {
                                              stopPropagation:false});
       Element.observe(elem, "blur", this.reuse.onblur);
       Element.observe(elem, "click", this.reuse.onclick);
+
+      // prevent browser builtin autocomplete behaviour
+      elem.setAttribute("autocomplete", "off");
     }
 
     // initialize time stamps
@@ -1885,12 +1889,14 @@ GvaScript.AutoCompleter.prototype = {
     // insert into DOM
     document.body.appendChild(div);
 
-    // simulate maxHeight/minWidth on MSIE (must be AFTER appendChild())
-    if (navigator.appVersion.match(/\bMSIE\b/)) {
+    // simulate maxHeight/minWidth on old MSIE (must be AFTER appendChild())
+    if (navigator.userAgent.match(/\bMSIE [456]\b/)) {
       div.style.setExpression("height", 
-        "this.scrollHeight>" + this.options.maxHeight + "?" + this.options.maxHeight + ":'auto'");
+        "this.scrollHeight>" + this.options.maxHeight + "?" 
+                             + this.options.maxHeight + ":'auto'");
       div.style.setExpression("width", 
-        "this.scrollWidth<" + this.options.minWidth + "?" + this.options.minWidth + ":'auto'");
+        "this.scrollWidth<" + this.options.minWidth + "?" 
+                            + this.options.minWidth + ":'auto'");
     }
 
     return this.dropdownDiv = div;
@@ -2028,6 +2034,10 @@ GvaScript.AutoCompleter.prototype = {
 
 
 //----------repeat.js
+/* TODO :
+    - invent syntax for IF blocks (first/last, odd/even)
+*/
+
 GvaScript.Repeat = {
 
 //-----------------------------------------------------
@@ -2038,50 +2048,80 @@ GvaScript.Repeat = {
     this._init_repeat_elements(elem);
   },
 
-  add: function(placeholder) {
+  add: function(repeat_name, count) {
+    if (count == undefined) count = 1;
 
-    // find the placeholder element
-    if (typeof placeholder == "string" && !placeholder.match(/.placeholder$/))
-        placeholder += ".placeholder";
-    placeholder = $(placeholder);
-
-    // repeat properties
-    var repeat = placeholder.repeat;
-
-    // won't add beyond max
-    if (repeat.count >= repeat.max)
-      return;
-
-    // increment the repetition block count and update path
-    var path_ini = repeat.path; // for restoring later; see end of block
-    repeat.ix    = ++repeat.count;
-    repeat.path  = repeat.path + "." + repeat.ix;
+    // get repeat properties
+    var placeholder = this._find_placeholder(repeat_name);
+    var repeat      = placeholder.repeat;
+    var path_ini    = repeat.path;
 
     // regex substitutions to build html for the new repetition block (can't
-    // do it with Template.replace() because working in only one namespace)
-    var regex = new RegExp("#{" + repeat.name + "\\.(\\w+)}", "g");
-    var repl  = function ($0, $1){return repeat[$1] || ""};
-    var html  = repeat.template.replace(regex, repl);
+    // use Template.replace() because we need structured namespaces)
+    var regex       = new RegExp("#{" + repeat.name + "\\.(\\w+)}", "g");
+    var replacement = function ($0, $1){var s = repeat[$1]; 
+                                        return s == undefined ? "" : s};
 
-    // insert into the DOM
-    placeholder.insert({before:html});
-    var insertion_block = $(repeat.path);
+    while (count-- > 0 && repeat.count < repeat.max) {
+      // increment the repetition block count and update path
+      repeat.ix    = repeat.count++;  // invariant: count == ix + 1
+      repeat.path  = path_ini + "." + repeat.ix;
+
+      // compute the HTML
+      var html  = repeat.template.replace(regex, replacement);
+
+      // insert into the DOM
+      placeholder.insert({before:html});
+      var insertion_block = $(repeat.path);
   
-    // repetition block gets an event
-    placeholder.fireEvent("Repeat", insertion_block);
+      // repetition block gets an event
+      placeholder.fireEvent("Add", insertion_block);
 
-    // deal with nested repeated sections
-    this._init_repeat_elements(insertion_block, repeat.path);
+      // deal with nested repeated sections
+      this._init_repeat_elements(insertion_block, repeat.path);
 
-    // restore initial path, because unlike Perl, JS has no "local" :-( 
-    repeat.path = path_ini;
+      // restore initial path
+      repeat.path = path_ini;
+    }
+
+    return repeat.count;
   },
 
+  remove: function(repetition_block) {
+    // find element, placeholder and repeat info
+    var elem = $(repetition_block);
+    elem.id.match(/(.*)\.(\d+)$/);
+    var repeat_name = RegExp.$1;
+    var remove_ix   = RegExp.$2;
+    var placeholder = this._find_placeholder(repeat_name);
+    var max         = placeholder.repeat.count;
+
+    // remove the repeat block and all blocks above
+    for (var i = remove_ix; i < max; i++) {
+      var block = $(repeat_name + "." + i);
+      placeholder.fireEvent("Remove", block);
+      block.remove();
+      placeholder.repeat.count -= 1;
+    }        
+
+    // add again the blocks above (which will be renumbered)
+    var n_add = max - remove_ix - 1;
+    if (n_add > 0) this.add(placeholder, n_add);
+  },
+
+
 
 //-----------------------------------------------------
 // Private methods
 //-----------------------------------------------------
 
+  _find_placeholder: function(name) {
+    if (typeof name == "string" && !name.match(/.placeholder$/))
+        name += ".placeholder";
+    var placeholder = $(name); 
+    if (!placeholder) throw new Error("no such element: " + name);
+    return placeholder;
+  },
 
   _init_repeat_elements: function(elem, path) {
     elem = $(elem);
@@ -2108,11 +2148,15 @@ GvaScript.Repeat = {
     element = $(element);
     path = path || element.getAttribute('repeat-prefix');
 
+    // number of initial repetition blocks
+    var n_blocks = element.getAttribute('repeat-start');
+    if (n_blocks == undefined) n_blocks = 1;
+
     // hash to hold all properties of the repeat element
     var repeat = {};
     repeat.name  = element.getAttribute('repeat');
-    repeat.min   = element.getAttribute('repeat-min') || 1,
-    repeat.max   = element.getAttribute('repeat-max') || 99,
+    repeat.min   = element.getAttribute('repeat-min') || 0;
+    repeat.max   = element.getAttribute('repeat-max') || 99;
     repeat.count = 0;
     repeat.path  = (path ? path + "." : "") + repeat.name;
 
@@ -2131,55 +2175,43 @@ GvaScript.Repeat = {
       element.id = "#{" + repeat.name + ".path}";
 
       // b) remove "repeat*" attributes (don't want them in the template)
-      for (var prop in element) {
-        if (prop.match(/^repeat/i))  element.removeAttribute(prop, 0);
+      var attrs = element.attributes;
+      var repeat_attrs = [];
+      for (var i = 0; i < attrs.length; i++) {
+        var name = attrs[i].name;
+        if (name.match(/^repeat/i)) repeat_attrs.push(name);
       }
+      repeat_attrs.each(function(name){element.removeAttribute(name, 0)});
 
-      // c) remove from DOM and keep it as a template string
-      // outerHTML would be simpler, but not supported by Gecko :-((
+      // c) keep it as a template string and remove from DOM
       repeat.template = Element.outerHTML(element);
       element.remove();
-
-//       var div = document.createElement("div");
-//       element = 
-//       div.appendChild(element);
-//       repeat.template = div.innerHTML;
-
     }
 
     // store all properties within the placeholder
     placeholder.repeat = repeat;
 
-    // initial repetition blocks 
-    var n_start = element.getAttribute('repeat-start') || repeat.min;
-    for (var i = 1; i <= n_start; i++) {
-      this.add(placeholder);
-    }
+    // create initial repetition blocks 
+    this.add(placeholder, n_blocks);
   }
 
 };
 
 //----------form.js
-/* 
-TODO
+/* TODO
 
-  - submit
-
-     enctype attr
-     after_submit:
-        - replace document
-        - replace values
+   - submit attrs on buttons
+       - action / method / enctype / replace / target / novalidate
+  - after_submit:
+        - 204 NO CONTENT : leave doc, apply metadata
+        - 205 RESET CONTENT : reset form
+        - replace="document" (new page)
+        - replace="values" (fill form with new tree)
         - relace element
         - others ?
-   - submit attrs on buttons
-       - action / method / enctype / replace / target
-
-  - "onreceive" event (response after submit)
-
-  - autofocus
+        - "onreceive" event (response after submit)
 
   - check prototype.js serialize on multivalues
-
 */
 
 GvaScript.Form = {
@@ -2188,9 +2220,7 @@ GvaScript.Form = {
     GvaScript.Repeat.init(form);
     if (tree)
       this.fill_from_tree(form, "", tree);
-
-    // TODO : enctype, onsubmit, etc.
-
+    this.autofocus(form);
   },
 
 
@@ -2206,9 +2236,6 @@ GvaScript.Form = {
 
   fill_from_tree : function(form, field_prefix, tree) {
     form = $(form);
-    if (!form)
-      return;
-
     for (var key in tree) {
       if (!tree.hasOwnProperty(key)) 
         continue;
@@ -2244,10 +2271,10 @@ GvaScript.Form = {
 
 
   _fill_from_array: function(form, field_prefix, array) {
-    for (var i=1; i < array.length; i++) {
+    for (var i=0; i < array.length; i++) {
       var new_prefix = field_prefix + "." + i;
 
-      // if form has a named element, fill it
+      // if form has a corresponding named element, fill it
       var elem = form[new_prefix];
       if (elem)
         this._fill_from_value(elem, array[i]);
@@ -2262,10 +2289,7 @@ GvaScript.Form = {
         if (!elem) { 
           var placeholder = $(field_prefix + ".placeholder");
           if (placeholder && placeholder.repeat) {
-            while (placeholder.repeat.count < i
-                   && placeholder.repeat.count < placeholder.repeat.max) {
-              GvaScript.Repeat.add(placeholder);
-            }
+            GvaScript.Repeat.add(placeholder, i + 1 - placeholder.repeat.count);
             elem = $(new_prefix);
           }
         }
@@ -2280,43 +2304,49 @@ GvaScript.Form = {
 
 
   _fill_from_value: function(elem, val) {
-    
-    if (!(val instanceof Array)) 
-      val = [val]; // force into an array
+        // IMPLEMENTATION NOTE : Form.Element.setValue() is quite similar,
+        // but our treatment of arrays is different, so we have to reimplement
 
-    if (elem.length !== undefined) { // if elem is a collection
-      for (var i=0; i < elem.length; i++) {
-        this._fill_from_value(elem.item(i), val);
-      }
-    }
-    else { // if elem is a single node
-      switch (elem.type) {
-        case "checkbox" :
-        case "radio":
-          elem.checked = val.include(elem.value);
-          break;
+    // force val into an array
+    if (!(val instanceof Array)) val = [val]; 
 
-        case "text" :
-        case "textarea" :
-        case "hidden" :
-          elem.value = val.join(",");
-          break;
+    // get element type (might be a node list, which we call "collection")
+    var elem_type = elem.type 
+                 || (elem.length !== undefined ? "collection" : "unknown");
 
-        case "select-one" :
-        case "select-multiple" :
-          $A(elem.options).each(function(opt){
-            opt.selected = val.include(opt.value);
-          });
-          break;
+    switch (elem_type) {
+      case "collection":
+        for (var i=0; i < elem.length; i++) {
+          this._fill_from_value(elem.item(i), val);
+        }
+        break;
 
-        default:
-          alert("unexpected elem type : " + elem.type);
-      } // end switch
-    } // end if
+      case "checkbox" :
+      case "radio":
+        elem.checked = val.include(elem.value);
+        break;
+
+      case "text" :
+      case "textarea" :
+      case "hidden" :
+        elem.value = val.join(",");
+        break;
+
+      case "select-one" :
+      case "select-multiple" :
+        $A(elem.options).each(function(opt){
+          var opt_value = Form.Element.Serializers.optionValue(opt);
+          opt.selected = val.include(opt_value);
+        });
+        break;
+
+      default:
+        throw new Error("unexpected elem type : " + elem.type);
+    } // end switch
   }, // end function
 
 
-  // javascript version of CGI::Expand::expand_hash
+  // javascript version of Perl  CGI::Expand::expand_hash
   expand_hash: function(flat_hash) {
     var tree = {};
 
@@ -2340,6 +2370,54 @@ GvaScript.Form = {
       loop.tree[loop.key] = flat_hash[k];
     }
     return tree.root;
+  }, 
+
+
+  add: function(repeat_name, count) {
+    var n_blocks = GvaScript.Repeat.add(repeat_name, count);
+    var last_block = repeat_name + "." + (n_blocks - 1);
+    this.autofocus(last_block);
+  },
+
+  remove: function(repetition_block) {
+    // find element and repeat info
+    var elem = $(repetition_block);
+    elem.id.match(/(.*)\.(\d+)$/);
+    var repeat_name = RegExp.$1;
+    var remove_ix   = RegExp.$2;
+    var form        = elem.up('form');
+
+    // get form data corresponding to the repeated section (should be an array)
+    var tree  = this.to_tree(form);
+    var parts = repeat_name.split(/\./);
+    for (var i = 0 ; i < parts.length; i++) {
+      if (!tree) break;
+      tree = tree[parts[i]];
+    }
+    
+    // remove rows below, and shift rows above
+    if (tree && tree instanceof Array) {
+      tree.splice(remove_ix, 1);
+      for (var i = 0 ; i < remove_ix; i++) {
+        delete tree[i];
+      }
+    }
+
+    // call Repeat.remove() to remove from DOM
+    GvaScript.Repeat.remove(repetition_block);
+
+    // re-populate blocks above
+    this.fill_from_tree(form, repeat_name, tree);
+  },
+
+  autofocus: function(elem) {
+    elem = $(elem);
+    if (elem) {
+      var target = elem.down('[autofocus]');
+      // TODO : check if target is visible
+      if (target) try {target.focus()} 
+                     catch(e){}
+    }
   }
 
 
